@@ -1,7 +1,7 @@
 import lightkurve as lk
 import numpy as np
 from scipy import sparse
-from tqdm.notebook import tqdm
+from tqdm import tqdm
 import warnings
 
 from astropy.timeseries import lombscargle
@@ -31,7 +31,7 @@ def vstack(dms):
         return lk.DesignMatrix(X, name=name, prior_mu=prior_mu, prior_sigma=prior_sigma)
 
 
-def SIP(tpfs, sigma=5, min_period=10, max_period=100, nperiods=300):
+def SIP(tpfs, sigma=5, min_period=10, max_period=100, nperiods=300, npca_components=2, aperture_threshold=3):
     """
     Systematics-insensitive periodogram for finding periods in long period NASA's TESS data.
 
@@ -56,6 +56,12 @@ def SIP(tpfs, sigma=5, min_period=10, max_period=100, nperiods=300):
         The maximum period for the periodogram
     nperiods : int
         The number of periods to fit
+    npca_components : int
+        Number of pca components to detrend with. Default is 2.
+    aperture_threshold : float
+        If there is no aperture mask from the pipeline, will create one. Set
+        aperture_threshold to set the thresholding for the aperture creation.
+        (See lightkurve's create_threshold_mask function.)
 
     Returns
     -------
@@ -66,17 +72,27 @@ def SIP(tpfs, sigma=5, min_period=10, max_period=100, nperiods=300):
             raw_lc: the original light curve from the input target pixel files
             corr_lc: the light curve with the best fitting systematics removed
             period_at_max_power: the best fit period of the sinusoid.
+            power_bkg: the power at each period for the pixels -outside- the aperture
+            raw_lc_bkg: the background light curve (pixels outside aperture)
     """
 
     # Get the un-background subtracted data
-    tpfs_uncorr = [(tpf + np.nan_to_num(tpf.flux_bkg.value))[np.isfinite(tpf.flux_bkg.value.sum(axis=(1, 2)))] for tpf in tpfs]
-    apers = [tpf.pipeline_mask for tpf in tpfs_uncorr]
-    lc = lk.LightCurveCollection([tpf.to_lightcurve(aperture_mask=aper) for tpf, aper in zip(tpfs_uncorr, apers)]).stitch()
+    if hasattr(tpfs[0], 'flux_bkg'):
+        tpfs_uncorr = [(tpf + np.nan_to_num(tpf.flux_bkg.value))[np.isfinite(tpf.flux_bkg.value.sum(axis=(1, 2)))] for tpf in tpfs]
+    else:
+        tpfs_uncorr = tpfs
+    apers = [tpf.pipeline_mask if tpf.pipeline_mask.any() else tpf.create_threshold_mask(aperture_threshold) for tpf in tpfs_uncorr]
+    lc = lk.LightCurveCollection([tpf.to_lightcurve(aperture_mask=aper) for tpf, aper in zip(tpfs_uncorr, apers)]).stitch(lambda x:x).normalize()
     lc.flux_err.value[~np.isfinite(lc.flux_err.value)] = np.nanmedian(lc.flux_err.value)
 
+    # Run the same routines on the background pixels
+    lc_bkg = lk.LightCurveCollection([tpf.to_lightcurve(aperture_mask=~aper) for tpf, aper in zip(tpfs_uncorr, apers)]).stitch(lambda x:x).normalize()
+    lc_bkg.flux_err.value[~np.isfinite(lc_bkg.flux_err.value)] = np.nanmedian(lc_bkg.flux_err.value)
+
+
     with warnings.catch_warnings():
         warnings.simplefilter('ignore')
-        bkgs = [lk.DesignMatrix(tpf.flux.value[:, ~aper], name='bkg').pca(3).append_constant().to_sparse() for tpf, aper in zip(tpfs_uncorr, apers)]
+        bkgs = [lk.DesignMatrix(tpf.flux.value[:, ~aper], name='bkg').pca(npca_components).append_constant().to_sparse() for tpf, aper in zip(tpfs_uncorr, apers)]
         for bkg in bkgs:
             bkg.prior_mu[-1] = 1
             bkg.prior_sigma[-1] = 0.1
@@ -89,7 +105,7 @@ def SIP(tpfs, sigma=5, min_period=10, max_period=100, nperiods=300):
     systematics_dm = vstack(bkgs)
 
     sigma_f_inv = sparse.csr_matrix(1/lc.flux_err.value[:, None]**2)
-    def fit_model(mask=None, return_model=False):
+    def fit_model(lc, mask=None, return_model=False):
         if mask is None:
             mask = np.ones(len(lc.flux.value), bool)
         sigma_w_inv = dm.X[mask].T.dot(dm.X[mask].multiply(sigma_f_inv[mask])).toarray()
@@ -98,7 +114,9 @@ def SIP(tpfs, sigma=5, min_period=10, max_period=100, nperiods=300):
         B = dm.X[mask].T.dot((lc.flux.value[mask]/lc.flux_err.value[mask]**2))
         B += dm.prior_mu/dm.prior_sigma**2
         w = np.linalg.solve(sigma_w_inv, B)
-        werr = ((np.linalg.inv(sigma_w_inv))**0.5).diagonal()
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore')
+            werr = ((np.linalg.inv(sigma_w_inv))**0.5).diagonal()
         if return_model:
             return dm.X.dot(w)
         return w, werr
@@ -108,24 +126,28 @@ def SIP(tpfs, sigma=5, min_period=10, max_period=100, nperiods=300):
     ls_dm = lk.DesignMatrix(lombscargle.implementations.mle.design_matrix(lc.time.jd, frequency=1/period, bias=False, nterms=1), name='LS').to_sparse()
     dm = lk.SparseDesignMatrixCollection([systematics_dm, ls_dm]).to_designmatrix(name='design_matrix')
 
-    # Do a first pass at 50 days, just to find ridiculous outliers
-    mod = fit_model(return_model=True)
-    mask = ~(lc - mod).remove_outliers(return_mask=True, sigma=sigma)[1]
+    # Do a first pass at 27 days, just to find ridiculous outliers
+    mod = fit_model(lc, return_model=True)
+    mask = ~(lc - mod*lc.flux.unit).remove_outliers(return_mask=True, sigma=sigma)[1]
 
     # Loop over some periods we care about
     periods = 1/np.linspace(1/min_period, 1/max_period, nperiods)
     ws = np.zeros((len(periods), dm.X.shape[1]))
     ws_err = np.zeros((len(periods), dm.X.shape[1]))
+    ws_bkg = np.zeros((len(periods), dm.X.shape[1]))
+    ws_err_bkg = np.zeros((len(periods), dm.X.shape[1]))
 
-    for idx, period in enumerate(tqdm(periods)):
+    for idx, period in enumerate(tqdm(periods, desc='Running pixels in aperture')):
         dm.X[:, -ls_dm.shape[1]:] = lombscargle.implementations.mle.design_matrix(lc.time.jd, frequency=1/period, bias=False, nterms=1)
-        ws[idx], ws_err[idx] = fit_model(mask=mask)
+        ws[idx], ws_err[idx] = fit_model(lc, mask=mask)
+        ws_bkg[idx], ws_err_bkg[idx] = fit_model(lc_bkg, mask=mask)
     power = (ws[:, -2]**2 + ws[:, -1]**2)**0.5
-
     am = np.argmax(power)
     dm.X[:, -ls_dm.shape[1]:] = lombscargle.implementations.mle.design_matrix(lc.time.jd, frequency=1/periods[am], bias=False, nterms=1)
     mod = dm.X[:, :-2].dot(ws[am][:-2])
 
+    power_bkg = (ws_bkg[:, -2]**2 + ws_bkg[:, -1]**2)**0.5
+
+    r = {'periods':periods, 'power':power, 'raw_lc':lc, 'power_bkg':power_bkg, 'raw_lc_bkg':lc_bkg, 'corr_lc':lc - mod*lc.flux.unit + 1, 'period_at_max_power':periods[am]}
 
-    r = {'periods':periods, 'power':power, 'raw_lc':lc, 'corr_lc':lc - mod + 1, 'period_at_max_power':periods[am]}
     return r
